@@ -19,8 +19,7 @@ static void derivative_huber(double *dk, double *ddk, double *r, double gamma, i
     if(fabs(r[i]) > gamma) {
       dk[i] = sign(r[i]);
       ddk[i] = 0.0;
-    } 
-    else {
+    } else {
       dk[i] = r[i]/gamma;
       ddk[i] = 1.0/gamma;
     }
@@ -31,8 +30,7 @@ static void derivative_quantapprox(double *dk, double *ddk, double *r, double ga
     if(fabs(r[i]) > gamma) {
       dk[i] = sign(r[i]);
       ddk[i] = 0.0;
-    } 
-    else {
+    } else {
       dk[i] = r[i]/gamma;
       ddk[i] = 1.0/gamma;
     }
@@ -133,14 +131,14 @@ static void unrescale(double *beta, double *colMin, double *colRange, int nlam,
 
 
 // Semismooth Newton Coordinate Descent (SNCD)
-static void sncd_huber(double *beta, int *iter, double *lambda, int *saturated, int *numv, double *x, double *y, double *psi, double *pf, double *gamma_, double *alpha_, double *eps_, double *lambda_min_, 
+static void sncd_huber(double *beta, int *iter, double *lambda, int *saturated, int *numv, double *x, double *y, double *d, double *pf, double *gamma_, double *alpha_, double *eps_, double *lambda_min_, 
 	int *nlam_, int *n_, int *p_, int *ppflag_, int *scrflag_, int *dfmax_, int *max_iter_, int *user_, int *message_)
 {
   // Declarations
   double gamma = gamma_[0]; double alpha = alpha_[0]; double eps = eps_[0]; double lambda_min = lambda_min_[0]; 
   int nlam = nlam_[0]; int n = n_[0]; int p = p_[0]; int ppflag = ppflag_[0]; int scrflag = scrflag_[0];
   int dfmax = dfmax_[0]; int max_iter = max_iter_[0]; int user = user_[0]; int message = message_[0];
-  int i, j, k, l, lp, jn, converged; double lstep, ldiff, lmax, l1, l2, v1, v2, v3, temp, shift, nullDev, max_update, update, thresh, strfactor = 1.0; 
+  int i, j, k, l, lp, jn, converged, mismatch; double pct, lstep, ldiff, lmax, l1, l2, v1, v2, v3, temp, shift, nullDev, max_update, update, thresh, strfactor = 1.0; 
   int nnzero = 0; // number of nonzero variables
   double *x2 = Calloc(n*p, double); // x^2
   for(i=0; i<n; i++) x2[i] = 1.0; // column of 1's for intercept
@@ -151,7 +149,7 @@ static void sncd_huber(double *beta, int *iter, double *lambda, int *saturated,
   double *beta_old = Calloc(p, double); 
   double *r = Calloc(n, double);
   double *w = Calloc(n, double);  
-  double *d = Calloc(p, double);
+  double *s = Calloc(p, double);
   double *dk = Calloc(n, double);
   double *ddk = Calloc(n, double); 
   double *z = Calloc(p, double); // partial derivative used for screening: X^t*dk/n
@@ -179,7 +177,7 @@ static void sncd_huber(double *beta, int *iter, double *lambda, int *saturated,
 
   // Initialization
   nullDev = 0.0; // not divided by n
-  for (int i=0;i<n;i++) {
+  for (i=0;i<n;i++) {
     r[i] = y[i];
     temp = fabs(r[i]);
     if(temp>gamma) {
@@ -189,7 +187,7 @@ static void sncd_huber(double *beta, int *iter, double *lambda, int *saturated,
     }
   }
   thresh = eps*nullDev;
-  //if (message) Rprintf("threshold = %f\n", thresh);
+  if (message) Rprintf("threshold = %f\n", thresh);
   derivative_huber(dk, ddk, r, gamma, n); 
  
   for(j=0; j<p; j++) {
@@ -198,12 +196,10 @@ static void sncd_huber(double *beta, int *iter, double *lambda, int *saturated,
 
   // Setup lambda
   if(user==0) {
-    lambda[0] = maxprod(x, psi, n, p, pf)/(n*alpha);
+    lambda[0] = maxprod(x, d, n, p, pf)/(n*alpha);
     if(lambda_min == 0.0) lambda_min = 0.001;
     lstep = log(lambda_min)/(nlam - 1);
     for(l=1; l<nlam; l++) lambda[l] = lambda[l-1]*exp(lstep);
-  } else {
-    for(l=0; l<nlam; l++) lambda[l] = lambda[l];
   }
 
   // Solution path
@@ -245,27 +241,27 @@ static void sncd_huber(double *beta, int *iter, double *lambda, int *saturated,
       // Solve KKT equations on eligible predictors
       while(iter[l]<max_iter) {
         iter[l]++;
-        max_update = 0.0; 
+        mismatch = 0; max_update = 0.0;
         for(j=0; j<p; j++) {
           if(ind[j]) {
             // Calculate v1, v2
-	    jn = j*n; v1 = 0.0; v2 = 0.0; 
+	    jn = j*n; v1 = 0.0; v2 = 0.0; pct = 0.0;
             for(i=0; i<n; i++) {
               v1 += x[jn+i]*dk[i];
               v2 += x2[jn+i]*ddk[i];
+              pct += ddk[i];
             }
-	    v1 = v1/n; v2 = v2/n;
-	    if(v2<0.01 && v2<1.0/n) {
-	      // Rprintf("Before: v2=%f\n", v2); 
-	      // small v2 causes numerical instability
+	    v1 = v1/n; v2 = v2/n; pct = pct*gamma/n;
+            //if(iter[l]==1 && j==1)Rprintf("l=%d, v1=%lf, v2=%lf, pct=%lf\n",l+1,v1,v2,pct);
+	    if(pct<0.05 || pct<1/n) {
 	      // approximate v2 with a continuation technique
               v2 = 0.0; 
 	      for(i=0; i<n; i++) {
-                if(r[i] > gamma) {
+		if (ddk[i]) {
+		  v2 += x2[jn+i]*ddk[i];
+		} else { // |r_i|>gamma
 		  w[i] = dk[i]/r[i];
                   v2 += x2[jn+i]*w[i];
-                } else {
-                  v2 += x2[jn+i]*ddk[i];
                 }
               }
               v2 = v2/n;
@@ -274,13 +270,18 @@ static void sncd_huber(double *beta, int *iter, double *lambda, int *saturated,
             // Update beta_j
             if(pf[j]==0.0) { // unpenalized
 	      beta[lp+j] = beta_old[j] + v1/v2; 
-            } else if(fabs(beta_old[j]+d[j])>1.0) { // active
-              d[j] = sign(beta_old[j]+d[j]);
-              beta[lp+j] = beta_old[j] + (v1-l1*pf[j]*d[j]-l2*pf[j]*beta_old[j])/(v2+l2*pf[j]); 
+            } else if(fabs(beta_old[j]+s[j])>1.0) { // active
+              s[j] = sign(beta_old[j]+s[j]);
+              beta[lp+j] = beta_old[j] + (v1-l1*pf[j]*s[j]-l2*pf[j]*beta_old[j])/(v2+l2*pf[j]); 
             } else {
-              d[j] = (v1+v2*beta_old[j])/(l1*pf[j]);
+              s[j] = (v1+v2*beta_old[j])/(l1*pf[j]);
               beta[lp+j] = 0.0;
             }
+            // mark the first mismatch between beta and s
+	    if (!mismatch && j>0) {
+              if (fabs(s[j]) > 1 || (beta[lp+j] != 0 && s[j] != sign(beta[lp+j])))
+		 mismatch = 1;
+            }
 	    // Update r, dk, ddk and compute candidate of max_update
             shift = beta[lp+j]-beta_old[j];
             if(shift!=0.0) {
@@ -305,7 +306,7 @@ static void sncd_huber(double *beta, int *iter, double *lambda, int *saturated,
         }
         // Check for convergence
         if(iter[l]>1) {
-          if(max_update < thresh) {
+          if(!mismatch && max_update < thresh) {
             converged = 1;
 	    break;
 	  }
@@ -323,7 +324,7 @@ static void sncd_huber(double *beta, int *iter, double *lambda, int *saturated,
 	      violations++;
 	      // pf[j] > 0
 	      // beta_old = beta = d = 0, no need for judgement
-              d[j] = v1/(l1*pf[j]);
+              s[j] = v1/(l1*pf[j]);
               if (violations == 1 & message) Rprintf("Lambda %d\n", l+1);
               if (message) Rprintf("+V%d", j);
 	    } else if (scrflag == 1 && ldiff != 0.0) {
@@ -362,21 +363,21 @@ static void sncd_huber(double *beta, int *iter, double *lambda, int *saturated,
   Free(beta_old);
   Free(r);
   Free(w);
-  Free(d);
+  Free(s);
   Free(dk);
   Free(ddk);
   Free(z);
   Free(ind);
 }
 
-static void sncd_quantile(double *beta, int *iter, double *lambda, int *saturated, int *numv, double *x, double *y, double *psi, double *pf, double *gamma_, double *tau_, double *alpha_, double *eps_, 
+static void sncd_quantile(double *beta, int *iter, double *lambda, int *saturated, int *numv, double *x, double *y, double *d, double *pf, double *gamma_, double *tau_, double *alpha_, double *eps_, 
        double *lambda_min_, int *nlam_, int *n_, int *p_, int *ppflag_, int *scrflag_, int *dfmax_, int *max_iter_, int *user_, int *message_)
 {
   // Declarations
   double gamma = gamma_[0]; double tau = tau_[0]; double c = 2*tau-1.0; double alpha = alpha_[0]; double eps = eps_[0]; double lambda_min = lambda_min_[0]; 
   int nlam = nlam_[0]; int n = n_[0]; int p = p_[0]; int ppflag = ppflag_[0]; int scrflag = scrflag_[0];
   int dfmax = dfmax_[0]; int max_iter = max_iter_[0]; int user = user_[0]; int message = message_[0];
-  int i, j, k, l, lp, jn, converged; double lstep, ldiff, lmax, l1, l2, v1, v2, v3, temp, shift, nullDev, max_update, update, thresh, strfactor = 1.0; 
+  int i, j, k, l, lp, jn, converged, mismatch; double pct, lstep, ldiff, lmax, l1, l2, v1, v2, v3, temp, shift, nullDev, max_update, update, thresh, strfactor = 1.0; 
   int nnzero = 0; // number of nonzero variables
   double *x2 = Calloc(n*p, double); // x^2
   for(i=0; i<n; i++) x2[i] = 1.0; // column of 1's for intercept
@@ -387,7 +388,7 @@ static void sncd_quantile(double *beta, int *iter, double *lambda, int *saturate
   double *beta_old = Calloc(p, double);
   double *r = Calloc(n, double);
   double *w = Calloc(n, double);
-  double *d = Calloc(p, double);
+  double *s = Calloc(p, double);
   double *dk = Calloc(n, double);
   double *ddk = Calloc(n, double);
   double *z = Calloc(p, double); // partial derivative used for screening: X^t*dk/n
@@ -416,7 +417,7 @@ static void sncd_quantile(double *beta, int *iter, double *lambda, int *saturate
 
   // Initialization
   nullDev = 0.0; // not divided by n
-  for (int i=0;i<n;i++) {
+  for (i=0;i<n;i++) {
     r[i] = y[i];
     temp = r[i];
     nullDev += fabs(temp) + c*temp;
@@ -427,23 +428,21 @@ static void sncd_quantile(double *beta, int *iter, double *lambda, int *saturate
 
   // Setup lambda
   if(user==0) {
-    lambda[0] = maxprod(x, psi, n, p, pf);
+    lambda[0] = maxprod(x, d, n, p, pf);
     // compute lambda[0] for original quantile loss
     for(i=0; i<n; i++) {
       if(fabs(r[i]) < 1e-10) {
-        psi[i] = 1.0+c;
+        d[i] = 1.0+c;
       } else {
-        psi[i] = sign(r[i])+c;
+        d[i] = sign(r[i])+c;
       } 
     }
-    temp = maxprod(x, psi, n, p, pf);
+    temp = maxprod(x, d, n, p, pf);
     if(temp>lambda[0]) lambda[0] = temp; // pick the larger one
     lambda[0] = lambda[0]/(2*n*alpha);
     if(lambda_min == 0.0) lambda_min = 0.001;
     lstep = log(lambda_min)/(nlam - 1);
     for(l=1; l<nlam; l++) lambda[l] = lambda[l-1]*exp(lstep);
-  } else {
-    for(l=0; l<nlam; l++) lambda[l] = lambda[l];
   }
 
   // Solution path
@@ -492,27 +491,27 @@ static void sncd_quantile(double *beta, int *iter, double *lambda, int *saturate
       // Solve KKT equations on eligible ones
       while(iter[l]<max_iter) {
         iter[l]++;
-        max_update = 0.0; 
+        mismatch = 0; max_update = 0.0;
         for(j=0; j<p; j++) {
           if(ind[j]) {
             // Calculate v1, v2
-	    jn = j*n; v1 = 0.0; v2 = 0.0; 
+	    jn = j*n; v1 = 0.0; v2 = 0.0; pct = 0.0;
             for(i=0; i<n; i++) {
               v1 += x[jn+i]*dk[i];
               v2 += x2[jn+i]*ddk[i];
+              pct += ddk[i];
             }
-	    v1 = v1/(2*n); v2 = v2/(2*n);
-	    if(v2<0.01 && v2<1.0/n) {
-	      //Rprintf("Before: v2=%f\n", v2);
-	      // small v2 causes numerical instability
+	    v1 = v1/(2*n); v2 = v2/(2*n); pct = pct*gamma/n;
+	    if(pct<0.05 || pct<1/n) {
+	      // Rprintf("j=%d, pct=%lf\n",j,pct);
 	      // approximate v2 with a continuation technique
               v2 = 0.0;
 	      for(i=0; i<n; i++) {
-                if(r[i] > 0.00001) {
+                if (ddk[i]) {
+                  v2 += x2[jn+i]*ddk[i];
+                } else { // |r_i| > gamma
 		  w[i] = (dk[i]-c)/r[i];
                   v2 += x2[jn+i]*w[i];
-                } else {
-                  v2 += x2[jn+i]*ddk[i];
                 }
               }
               v2 = v2/(2*n);
@@ -521,13 +520,18 @@ static void sncd_quantile(double *beta, int *iter, double *lambda, int *saturate
             // Update beta_j
             if(pf[j]==0.0) { // unpenalized
 	      beta[lp+j] = beta_old[j] + v1/v2;
-            } else if(fabs(beta_old[j]+d[j])>1.0) { // active
-              d[j] = sign(beta_old[j]+d[j]);
-              beta[lp+j] = beta_old[j] + (v1-l1*pf[j]*d[j]-l2*pf[j]*beta_old[j])/(v2+l2*pf[j]);
+            } else if(fabs(beta_old[j]+s[j])>1.0) { // active
+              s[j] = sign(beta_old[j]+s[j]);
+              beta[lp+j] = beta_old[j] + (v1-l1*pf[j]*s[j]-l2*pf[j]*beta_old[j])/(v2+l2*pf[j]);
             } else {
-              d[j] = (v1+v2*beta_old[j])/(l1*pf[j]);
+              s[j] = (v1+v2*beta_old[j])/(l1*pf[j]);
               beta[lp+j] = 0.0;
             }
+            // mark the first mismatch between beta and s
+	    if (!mismatch && j>0) {
+              if (fabs(s[j]) > 1 || (beta[lp+j] != 0 && s[j] != sign(beta[lp+j])))
+		 mismatch = 1;
+            }
 	    // Update r, dk, ddk and compute candidate of max_update
             shift = beta[lp+j]-beta_old[j];
             if(shift!=0) {
@@ -554,8 +558,7 @@ static void sncd_quantile(double *beta, int *iter, double *lambda, int *saturate
         }
         // Check for convergence
         if(iter[l]>10) {
-	  //if(l==0) Rprintf("max_update = %lf\n", max_update);
-          if(max_update < thresh) {
+          if(!mismatch && max_update < thresh) {
             converged = 1;
 	    break;
 	  }
@@ -573,7 +576,7 @@ static void sncd_quantile(double *beta, int *iter, double *lambda, int *saturate
 	      violations++;
 	      // pf[j] > 0
 	      // beta_old = beta = d = 0, no need for judgement
-              d[j] = v1/(l1*pf[j]);
+              s[j] = v1/(l1*pf[j]);
               if (message) Rprintf("+V%d", j);
 	    } else if (scrflag == 1 && ldiff != 0.0) {
 	      v3 = fabs((v1-z[j])/(pf[j]*ldiff*alpha));
@@ -611,21 +614,21 @@ static void sncd_quantile(double *beta, int *iter, double *lambda, int *saturate
   Free(beta_old);
   Free(r);
   Free(w);
-  Free(d);
+  Free(s);
   Free(dk);
   Free(ddk);
   Free(z);
   Free(ind);
 }
 
-static void sncd_squared(double *beta, int *iter, double *lambda, int *saturated, int *numv, double *x, double *y, double *psi, double *pf, double *alpha_, double *eps_, double *lambda_min_, 
+static void sncd_squared(double *beta, int *iter, double *lambda, int *saturated, int *numv, double *x, double *y, double *d, double *pf, double *alpha_, double *eps_, double *lambda_min_, 
 	int *nlam_, int *n_, int *p_, int *ppflag_, int *scrflag_, int *dfmax_, int *max_iter_, int *user_, int *message_)
 {
   // Declarations
   double alpha = alpha_[0]; double eps = eps_[0]; double lambda_min = lambda_min_[0]; 
   int nlam = nlam_[0]; int n = n_[0]; int p = p_[0]; int ppflag = ppflag_[0]; int scrflag = scrflag_[0];
   int dfmax = dfmax_[0]; int max_iter = max_iter_[0]; int user = user_[0]; int message = message_[0];
-  int i, j, k, l, lp, jn, converged; double lstep, ldiff, lmax, l1, l2, v1, v2, v3, temp, shift, nullDev, max_update, update, thresh, strfactor = 1.0;
+  int i, j, k, l, lp, jn, converged, mismatch; double lstep, ldiff, lmax, l1, l2, v1, v2, v3, temp, shift, nullDev, max_update, update, thresh, strfactor = 1.0;
   int nnzero = 0; // number of nonzero variables
   double *x2 = Calloc(n*p, double); // x^2
   for(i=0; i<n; i++) x2[i] = 1.0; // column of 1's for intercept
@@ -636,8 +639,8 @@ static void sncd_squared(double *beta, int *iter, double *lambda, int *saturated
   double *colMin = Calloc(p, double);
   double *colRange = Calloc(p, double);
   double *beta_old = Calloc(p, double); 
-  double *r = Calloc(n, double); 
-  double *d = Calloc(p, double);
+  double *r = Calloc(n, double);
+  double *s = Calloc(p, double);
   double *z = Calloc(p, double); // X^t * r/n
   double cutoff;
   int *ind = Calloc(p, int);
@@ -677,12 +680,10 @@ static void sncd_squared(double *beta, int *iter, double *lambda, int *saturated
   
   // Setup lambda
   if(user==0) {
-    lambda[0] = maxprod(x, psi, n, p, pf)/(n*alpha);
+    lambda[0] = maxprod(x, d, n, p, pf)/(n*alpha);
     if(lambda_min == 0.0) lambda_min = 0.001;
     lstep = log(lambda_min)/(nlam - 1);
     for(l=1; l<nlam; l++) lambda[l] = lambda[l-1]*exp(lstep);
-  } else {
-    for(l=0; l<nlam; l++) lambda[l] = lambda[l];
   }
 
   // Solution path
@@ -724,7 +725,7 @@ static void sncd_squared(double *beta, int *iter, double *lambda, int *saturated
       // Solve KKT equations on eligible ones
       while(iter[l]<max_iter) {
         iter[l]++;
-	max_update = 0.0;
+	mismatch=0; max_update = 0.0;
         for(j=0; j<p; j++) {
           if(j == 0 && ppflag == 1) continue; // intercept is constant for standardized data
           if(ind[j]) {
@@ -733,14 +734,18 @@ static void sncd_squared(double *beta, int *iter, double *lambda, int *saturated
             // Update beta_j
             if(pf[j]==0.0) { // unpenalized
 	      beta[lp+j] = beta_old[j] + v1/v2;
-	    } else if(fabs(beta_old[j]+d[j])>1.0) { // active
-              d[j] = sign(beta_old[j]+d[j]);
-              beta[lp+j] = beta_old[j] + (v1-l2*pf[j]*beta_old[j]-l1*pf[j]*d[j])/(v2+l2*pf[j]);
+	    } else if(fabs(beta_old[j]+s[j])>1.0) { // active
+              s[j] = sign(beta_old[j]+s[j]);
+              beta[lp+j] = beta_old[j] + (v1-l2*pf[j]*beta_old[j]-l1*pf[j]*s[j])/(v2+l2*pf[j]);
             } else {
-              d[j] = (v1+v2*beta_old[j])/(l1*pf[j]);
+              s[j] = (v1+v2*beta_old[j])/(l1*pf[j]);
               beta[lp+j] = 0.0;
             }
-
+            // mark the first mismatch between beta and s
+	    if (!mismatch && j>0) {
+              if (fabs(s[j]) > 1 || (beta[lp+j] != 0 && s[j] != sign(beta[lp+j])))
+		 mismatch = 1;
+            }
 	    // Update residuals
             shift = beta[lp+j]-beta_old[j];       
             if(shift!=0.0) {
@@ -754,7 +759,7 @@ static void sncd_squared(double *beta, int *iter, double *lambda, int *saturated
         }             
         // Check for convergence
         if(iter[l]>1) {
-          if(max_update < thresh) {
+          if(!mismatch && max_update < thresh) {
             converged = 1;
 	    break;
 	  }
@@ -772,7 +777,7 @@ static void sncd_squared(double *beta, int *iter, double *lambda, int *saturated
 	      violations++;
 	      // pf[j] > 0
 	      // beta_old = beta = d = 0, no need for judgement
-              d[j] = v1/(l1*pf[j]);
+              s[j] = v1/(l1*pf[j]);
               if (violations == 1 & message) Rprintf("Lambda %d\n", l+1);
               if (message) Rprintf("+V%d", j);
 	    } else if (scrflag == 1 && ldiff != 0.0) {
@@ -810,14 +815,414 @@ static void sncd_squared(double *beta, int *iter, double *lambda, int *saturated
   Free(colRange);
   Free(beta_old);
   Free(r);
-  Free(d);
+  Free(s);
   Free(ind);
 }
 
+// alpha = 0, pure l2 penalty
+static void sncd_huber_l2(double *beta, int *iter, double *lambda, double *x, double *y, double *d, double *pf, double *gamma_, double *eps_, double *lambda_min_, 
+	int *nlam_, int *n_, int *p_, int *ppflag_, int *max_iter_, int *user_, int *message_)
+{
+  // Declarations
+  double gamma = gamma_[0]; double eps = eps_[0]; double lambda_min = lambda_min_[0]; 
+  int nlam = nlam_[0]; int n = n_[0]; int p = p_[0]; int ppflag = ppflag_[0];
+  int max_iter = max_iter_[0]; int user = user_[0]; int message = message_[0];
+  int i, j, k, l, lp, jn, converged; double pct, lstep, ldiff, lmax, v1, v2, v3, temp, shift, nullDev, max_update, update, thresh;
+  double *x2 = Calloc(n*p, double); // x^2
+  for(i=0; i<n; i++) x2[i] = 1.0; // column of 1's for intercept
+  double *center = Calloc(p, double);
+  double *scale = Calloc(p, double);
+  double *colMin = Calloc(p, double);
+  double *colRange = Calloc(p, double);
+  double *beta_old = Calloc(p, double); 
+  double *r = Calloc(n, double);
+  double *w = Calloc(n, double);  
+  double *dk = Calloc(n, double);
+  double *ddk = Calloc(n, double);
+  
+  // Preprocessing
+  if (ppflag == 1) {
+    standardize(x, x2, center, scale, n, p);
+  } else if (ppflag == 2) {
+    rescale(x, x2, colMin, colRange, n, p);
+  } else {
+    for(j=1; j<p; j++) {
+      jn = j*n;
+      for(i=0; i<n; i++) x2[jn+i]=pow(x[jn+i],2);
+    }
+  }
+
+  // Initialization
+  nullDev = 0.0; // not divided by n
+  for (i=0;i<n;i++) {
+    r[i] = y[i];
+    temp = fabs(r[i]);
+    if(temp>gamma) {
+      nullDev += temp - gamma/2;
+    } else {
+      nullDev += temp*temp/(2*gamma);
+    }
+  }
+  thresh = eps*nullDev;
+  //if (message) Rprintf("threshold = %f\n", thresh);
+  derivative_huber(dk, ddk, r, gamma, n); 
+
+  // Setup lambda
+  if(user==0) {
+    lambda[0] = maxprod(x, d, n, p, pf)/(n*0.01);
+    if(lambda_min == 0.0) lambda_min = 0.001;
+    lstep = log(lambda_min)/(nlam - 1);
+    for(l=1; l<nlam; l++) lambda[l] = lambda[l-1]*exp(lstep);
+  }
+
+  // Solution path
+  for(l=0; l<nlam; l++) {
+    converged = 0; lp = l*p;
+    while(iter[l] < max_iter) {
+      iter[l]++;
+      max_update = 0.0; 
+      for(j=0; j<p; j++) {
+        // Calculate v1, v2
+	jn = j*n; v1 = 0.0; v2 = 0.0; pct = 0.0;
+        for(i=0; i<n; i++) {
+          v1 += x[jn+i]*dk[i];
+          v2 += x2[jn+i]*ddk[i];
+          pct += ddk[i];
+        }
+	v1 = v1/n; v2 = v2/n; pct = pct*gamma/n;
+	if(pct<0.05 || pct<1/n) {
+          //Rprintf("j=%d, pct=%lf\n",j,pct);
+	  // approximate v2 with a continuation technique
+          v2 = 0.0; 
+	  for(i=0; i<n; i++) {
+	    if (ddk[i]) {
+	      v2 += x2[jn+i]*ddk[i];
+	    } else { // |r_i|>gamma
+              w[i] = dk[i]/r[i];
+              v2 += x2[jn+i]*w[i];
+            }
+          }
+          v2 = v2/n;
+          // Rprintf("After: v2=%f\n", v2);              
+	}
+        // Update beta_j
+        if(pf[j]==0.0) { // unpenalized
+	  beta[lp+j] = beta_old[j] + v1/v2; 
+        } else {
+          beta[lp+j] = beta_old[j] + (v1-lambda[l]*pf[j]*beta_old[j])/(v2+lambda[l]*pf[j]); 
+        }
+	// Update r, dk, ddk and compute candidate of max_update
+        shift = beta[lp+j]-beta_old[j];
+        if(shift!=0.0) {
+	  v2 = 0.0;
+          for(i=0; i<n; i++) {
+	    r[i] -= x[jn+i]*shift;
+            if(fabs(r[i])>gamma) {
+              dk[i] = sign(r[i]);
+              ddk[i] = 0.0;
+            } else {
+	      dk[i] = r[i]/gamma;
+              ddk[i] = 1.0/gamma;
+	      v2 += x2[jn+i]*ddk[i];
+	    }
+	  }
+	  v2 += n*lambda[l]*pf[j];
+	  update = v2*shift*shift;
+          if(update>max_update) max_update = update;
+          beta_old[j] = beta[lp+j];
+        }
+      }
+      // Check for convergence
+      if(iter[l]>1) {
+        if(max_update < thresh) {
+          converged = 1;
+	  break;
+	}
+      }
+    }
+  }
+  // Postprocessing
+  if (ppflag == 1) {
+    unstandardize(beta, center, scale, nlam, p);
+  } else if (ppflag == 2) {
+    unrescale(beta, colMin, colRange, nlam, p);
+  }
+
+  Free(x2);
+  Free(center);
+  Free(scale);
+  Free(colMin);
+  Free(colRange);
+  Free(beta_old);
+  Free(r);
+  Free(w);
+  Free(dk);
+  Free(ddk);
+}
+
+static void sncd_quantile_l2(double *beta, int *iter, double *lambda, double *x, double *y, double *d, double *pf, double *gamma_, double *tau_, double *eps_, 
+       double *lambda_min_, int *nlam_, int *n_, int *p_, int *ppflag_, int *max_iter_, int *user_, int *message_)
+{
+  // Declarations
+  double gamma = gamma_[0]; double tau = tau_[0]; double c = 2*tau-1.0; double eps = eps_[0]; double lambda_min = lambda_min_[0]; 
+  int nlam = nlam_[0]; int n = n_[0]; int p = p_[0]; int ppflag = ppflag_[0];
+  int max_iter = max_iter_[0]; int user = user_[0]; int message = message_[0];
+  int i, j, k, l, lp, jn, converged; double pct, lstep, ldiff, lmax, v1, v2, v3, temp, shift, nullDev, max_update, update, thresh;
+  double *x2 = Calloc(n*p, double); // x^2
+  for(i=0; i<n; i++) x2[i] = 1.0; // column of 1's for intercept
+  double *center = Calloc(p, double);
+  double *scale = Calloc(p, double);
+  double *colMin = Calloc(p, double);
+  double *colRange = Calloc(p, double);
+  double *beta_old = Calloc(p, double); 
+  double *r = Calloc(n, double);
+  double *w = Calloc(n, double);  
+  double *dk = Calloc(n, double);
+  double *ddk = Calloc(n, double);
+  int m = (int) (n*0.10);
+
+  // Preprocessing
+  if (ppflag == 1) {
+    standardize(x, x2, center, scale, n, p);
+  } else if (ppflag == 2) {
+    rescale(x, x2, colMin, colRange, n, p);
+  } else {
+    for(j=1; j<p; j++) {
+      jn = j*n;
+      for(i=0; i<n; i++) x2[jn+i]=pow(x[jn+i],2);
+    }
+  }
+
+  // Initialization
+  nullDev = 0.0; // not divided by n
+  for (i=0;i<n;i++) {
+    r[i] = y[i];
+    temp = r[i];
+    nullDev += fabs(temp) + c*temp;
+  }
+  thresh = eps*nullDev;
+  derivative_quantapprox(dk, ddk, r, gamma, c, n);
+
+  // Setup lambda
+  if(user==0) {
+    lambda[0] = maxprod(x, d, n, p, pf);
+    // compute lambda[0] for original quantile loss
+    for(i=0; i<n; i++) {
+      if(fabs(r[i]) < 1e-10) {
+        d[i] = 1.0+c;
+      } else {
+        d[i] = sign(r[i])+c;
+      } 
+    }
+    temp = maxprod(x, d, n, p, pf);
+    if(temp>lambda[0]) lambda[0] = temp; // pick the larger one
+    lambda[0] = lambda[0]/(2*n*0.01);
+    if(lambda_min == 0.0) lambda_min = 0.001;
+    lstep = log(lambda_min)/(nlam - 1);
+    for(l=1; l<nlam; l++) lambda[l] = lambda[l-1]*exp(lstep);
+  }
+
+  // Solution path
+  for(l=0; l<nlam; l++) {
+    if(gamma>0.001 && l>0) {
+      temp = ksav(r, n, m);
+      if (temp < gamma) gamma = temp;
+    }
+    if(gamma<0.001) gamma = 0.001;
+    if (message) Rprintf("Lambda %d: Gamma = %f\n", l+1, gamma);
+    converged = 0; lp = l*p;
+    while(iter[l] < max_iter) {
+      iter[l]++;
+      max_update = 0.0; 
+      for(j=0; j<p; j++) {
+        // Calculate v1, v2
+	jn = j*n; v1 = 0.0; v2 = 0.0; pct = 0.0;
+        for(i=0; i<n; i++) {
+          v1 += x[jn+i]*dk[i];
+          v2 += x2[jn+i]*ddk[i];
+          pct += ddk[i];
+        }
+	v1 = v1/(2*n); v2 = v2/(2*n); pct = pct*gamma/n;
+	if(pct<0.05 || pct<1/n) {
+	  // Rprintf("j=%d, pct=%lf\n",j,pct);
+	  // approximate v2 with a continuation technique
+          v2 = 0.0;
+	  for(i=0; i<n; i++) {
+            if (ddk[i]) {
+              v2 += x2[jn+i]*ddk[i];
+            } else { // |r_i| > gamma
+	      w[i] = (dk[i]-c)/r[i];
+              v2 += x2[jn+i]*w[i];
+            }
+          }
+          v2 = v2/(2*n);
+          //Rprintf("After: v2=%f\n", v2);
+	}
+        // Update beta_j
+        if(pf[j]==0.0) { // unpenalized
+	  beta[lp+j] = beta_old[j] + v1/v2; 
+        } else {
+          beta[lp+j] = beta_old[j] + (v1-lambda[l]*pf[j]*beta_old[j])/(v2+lambda[l]*pf[j]); 
+        }
+	// Update r, dk, ddk and compute candidate of max_update
+        shift = beta[lp+j]-beta_old[j];
+        if(shift!=0) {
+	  v2 = 0.0;
+          for(i=0; i<n; i++) {
+	    r[i] -= x[jn+i]*shift;
+            if(fabs(r[i])>gamma) {
+              dk[i] = sign(r[i])+c;
+              ddk[i] = 0.0;
+            } else {
+	      dk[i] = r[i]/gamma+c;
+              ddk[i] = 1.0/gamma;
+	    }
+	  }
+          update = n*(v2*shift*shift + 2*fabs(v1*shift));
+          if(update>max_update) max_update = update;
+          beta_old[j] = beta[lp+j];
+        }
+      }
+      // Check for convergence
+      if(iter[l]>10) {
+        if(max_update < thresh) {
+          converged = 1;
+	  break;
+	}
+      }
+    }
+  }
+  // Postprocessing
+  if (ppflag == 1) {
+    unstandardize(beta, center, scale, nlam, p);
+  } else if (ppflag == 2) {
+    unrescale(beta, colMin, colRange, nlam, p);
+  }
+
+  Free(x2);
+  Free(center);
+  Free(scale);
+  Free(colMin);
+  Free(colRange);
+  Free(beta_old);
+  Free(r);
+  Free(w);
+  Free(dk);
+  Free(ddk);
+}
+
+static void sncd_squared_l2(double *beta, int *iter, double *lambda, double *x, double *y, double *d, double *pf, double *eps_, double *lambda_min_, 
+	int *nlam_, int *n_, int *p_, int *ppflag_, int *max_iter_, int *user_, int *message_)
+{
+  // Declarations
+  double eps = eps_[0]; double lambda_min = lambda_min_[0]; 
+  int nlam = nlam_[0]; int n = n_[0]; int p = p_[0]; int ppflag = ppflag_[0];
+  int max_iter = max_iter_[0]; int user = user_[0]; int message = message_[0];
+  int i, j, k, l, lp, jn, converged; double pct, lstep, ldiff, lmax, v1, v2, v3, temp, shift, nullDev, max_update, update, thresh;
+  double *x2 = Calloc(n*p, double); // x^2
+  for(i=0; i<n; i++) x2[i] = 1.0; // column of 1's for intercept
+  double *x2bar = Calloc(p, double); // Col Mean of x2
+  x2bar[0] = 1.0; 
+  double *center = Calloc(p, double);
+  double *scale = Calloc(p, double);
+  double *colMin = Calloc(p, double);
+  double *colRange = Calloc(p, double);
+  double *beta_old = Calloc(p, double); 
+  double *r = Calloc(n, double);
+  
+  // Preprocessing
+  if (ppflag == 1) {
+    standardize(x, x2, center, scale, n, p);
+  } else if (ppflag == 2) {
+    rescale(x, x2, colMin, colRange, n, p);
+  } else {
+    for(j=1; j<p; j++) {
+      jn = j*n;
+      for(i=0; i<n; i++) x2[jn+i]=pow(x[jn+i],2);
+    }
+  }
+
+  // Initialization
+  nullDev = 0.0;
+  for(i=0; i<n; i++) {
+    r[i] = y[i];
+    nullDev += pow(r[i],2); // without dividing by 2n
+  }
+  thresh = eps*nullDev;
+  for(j=0; j<p; j++) {
+    jn = j*n;
+    for(i=0; i<n; i++) x2bar[j] += x2[jn+i];
+    x2bar[j] = x2bar[j]/n;
+  }
+
+  // Setup lambda
+  if(user==0) {
+    lambda[0] = maxprod(x, d, n, p, pf)/(n*0.01);
+    if(lambda_min == 0.0) lambda_min = 0.001;
+    lstep = log(lambda_min)/(nlam - 1);
+    for(l=1; l<nlam; l++) lambda[l] = lambda[l-1]*exp(lstep);
+  }
+
+  // Solution path
+  for(l=0; l<nlam; l++) {
+    converged = 0; lp = l*p;
+    while(iter[l] < max_iter) {
+      iter[l]++;
+      max_update = 0.0; 
+      for(j=0; j<p; j++) {
+        if(j == 0 && ppflag == 1) continue; // intercept is constant for standardized data
+        // Update v1, v2=x2bar[j]
+      	v1 = crossprod(x, r, n, j)/n; v2 = x2bar[j];
+        // Update beta_j
+        if(pf[j]==0.0) { // unpenalized
+	  beta[lp+j] = beta_old[j] + v1/v2;
+	} else {
+          beta[lp+j] = beta_old[j] + (v1-lambda[l]*pf[j]*beta_old[j])/(v2+lambda[l]*pf[j]);
+        }
+	// Update r
+        shift = beta[lp+j]-beta_old[j];       
+        if(shift!=0.0) {
+	  jn = j*n;              
+          for(i=0; i<n; i++) r[i] -= x[jn+i]*shift;
+	  update = n*(1.0+lambda[l]*pf[j])*shift*shift;
+	  if(update>max_update) max_update = update;
+	  beta_old[j] = beta[lp+j];
+        }
+      }
+      // Check for convergence
+      if(iter[l]>1) {
+        if(max_update < thresh) {
+          converged = 1;
+	  break;
+	}
+      }
+    }
+  }
+  // Postprocessing
+  if (ppflag == 1) {
+    unstandardize(beta, center, scale, nlam, p);
+  } else if (ppflag == 2) {
+    unrescale(beta, colMin, colRange, nlam, p);
+  }
+
+  Free(x2);
+  Free(x2bar);
+  Free(center);
+  Free(scale);
+  Free(colMin);
+  Free(colRange);
+  Free(beta_old);
+  Free(r);
+}
+
+
 static const R_CMethodDef cMethods[] = {
   {"huber", (DL_FUNC) &sncd_huber, 22},
   {"quant", (DL_FUNC) &sncd_quantile, 23},
   {"squared", (DL_FUNC) &sncd_squared, 21},
+  {"huber_l2", (DL_FUNC) &sncd_huber_l2, 17},
+  {"quantile_l2", (DL_FUNC) &sncd_quantile_l2, 18},
+  {"squared_l2", (DL_FUNC) &sncd_squared_l2, 16},
   {NULL}
 };
 
